module CGameController;
import std.math;
import DOpenGL.gl;
import Obelisk.CColour, Obelisk.CVector, Obelisk.EKey, Obelisk.IObelisk;
import CSideScrollerMode, CTopDownMode, CTownMode, IGameMode;

class CGameController
{
	private IObelisk mObelisk;
	private IGameMode mCurrentMode;
	private string mTextboxText;
	private string mTextboxAvatar;
	private bool mGameOver = false;
	private bool mWin = false;
	private int mCrazyTime = -1;
	private int mCrazyTime1 = 0;

	private int mCheckpoint;

	public this(IObelisk aObelisk)
	{
		mObelisk = aObelisk;

		GameStart();
	}



	public void Update()
	{
		if (mWin)
			return;
		if (mGameOver)
		{
			if (mObelisk.IsKeyPressed(EKey.Enter))
			{
				mGameOver = false;
				GameUpdate();
				return;
			}
		}
		if (mTextboxText != "" || mCrazyTime != -1)
		{
			if (mObelisk.IsKeyPressed(EKey.Enter))
			{
				mTextboxText = "";
				mCrazyTime = -1;
				GameNext();
			}
		}
		else
			mCurrentMode.Update(this);
	}



	public void Render()
	{
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		if (mGameOver)
		{
			mObelisk.TextBegin(400, 270, 0.5);
			mObelisk.TextSetFont("Berylium.ttf", 140);
			mObelisk.TextWrite("Game Over");
			mObelisk.TextEnd();

			mObelisk.TextBegin(400, 400, 0.5, 30);
			mObelisk.TextSetFont("Berylium.ttf", 50);
			mObelisk.TextWrite("Press Enter to retry\n");
			mObelisk.TextWrite("Press Escape to quit");
			mObelisk.TextEnd();
			return;
		}
		if (mWin)
		{
			mObelisk.TextBegin(400, 270, 0.5);
			mObelisk.TextSetFont("Berylium.ttf", 140);
			mObelisk.TextWrite("You Win!");
			mObelisk.TextEnd();

			mObelisk.TextBegin(400, 400, 0.5, 30);
			mObelisk.TextSetFont("Berylium.ttf", 50);
			mObelisk.TextWrite("Press Escape to quit");
			mObelisk.TextEnd();
			return;
		}

		mCurrentMode.Render();
		if (mTextboxText != "")
		{
			mObelisk.DrawSprite(mObelisk.GetTexture("Textbox.png"), 19, 600 - 19 - 141);
			mObelisk.DrawSprite(mObelisk.GetTexture(mTextboxAvatar), 34, 462);
			mObelisk.TextBegin(112, 600 - 19 - 141 + 50, 0, 20);
			mObelisk.TextSetFont("Berylium.ttf", 35);
			mObelisk.TextWrite(mTextboxText);
			mObelisk.TextEnd();
		}
		if (mCrazyTime != -1)
		{
			float Hue = 6 * (1 - (cos(mCrazyTime * (PI * 2) / (50 * 3)) / 2 + 0.5));
			float R = 0;
			float G = 0;
			float B = 0;
			if (Hue >= 0 && Hue <= 1)   // red -> yellow
			{
				R = 1;
				G = Interpolate(0, 1, Hue, 1);
				B = 0;
			}
			else if (Hue > 1 && Hue <= 2)   // yellow -> green
			{
				R = Interpolate(1, 0, Hue - 1, 1);
				G = 1;
				B = 0;
			}
			else if (Hue > 2 && Hue <= 3)   // green -> cyan
			{
				R = 0;
				G = 1;
				B = Interpolate(0, 1, Hue - 2, 1);
			}
			else if (Hue > 3 && Hue <= 4)   // cyan -> blue
			{
				R = 0;
				G = Interpolate(1, 0, Hue - 3, 1);
				B = 1;
			}
			else if (Hue > 4 && Hue <= 5)   // blue -> magenta
			{
				R = Interpolate(0, 1, Hue - 4, 1);
				G = 0;
				B = 1;
			}
			else if (Hue > 5 && Hue <= 6)   // magenta -> red
			{
				R = 1;
				G = 0;
				B = Interpolate(1, 0, Hue - 5, 1);
			}
			mObelisk.SetAlpha(Interpolate(0.5, 0.1, cos(mCrazyTime * (PI * 2) / (50 * 2)) / 2 + 0.5, 1));
			mObelisk.QuadColour(new CVector(0, 0), new CVector(800, 0), new CVector(800, 600),
					new CVector(0, 600), new CColour(R, G, B));
			mObelisk.SetAlpha(1);
			mCrazyTime = (mCrazyTime + 1) % (50 * 6);
			if (++mCrazyTime1 == 5 * 50)
			{
				mCrazyTime = -1;
				mCrazyTime1 = 0;
				GameNext();
			}
		}
	}



	private float Interpolate(float aV0, float aV1, float aNumerator, float aDenominator)
	{
		float aDelta = aNumerator / aDenominator;
		return aV0 * (1 - aDelta) + aV1 * aDelta;
	}



	private void GameStart()
	{
		mCheckpoint = 0;
		GameUpdate();
	}



	public void GameNext()
	{
		mCheckpoint++;
		GameUpdate();
	}



	private void GameUpdate()
	{
		final switch (mCheckpoint)
		{
			case 0:
				mCurrentMode = new CTownMode(mObelisk, "Town.png", "Act 1");
				break;
			case 1:
				mTextboxText = "More monsters are approaching! Get out there\nand stop them!";
				mTextboxAvatar = "Avatar person.png";
				break;
			case 2:
				mCurrentMode = CSideScrollerMode.TownCombat(mObelisk, 8, true, [
					new CBackgroundObject(400, 370, "Bkg town.png")
				]);
				break;
			case 3:
				mCurrentMode = new CTownMode(mObelisk, "Town.png");
				mTextboxText = "These monster attacks get stronger every time. I\ndon't think we can hold out much longer.";
				mTextboxAvatar = "Avatar person.png";
				break;
			case 4:
				mTextboxText = "Go to the King and ask for help.";
				break;
			case 5:
				mCurrentMode = CSideScrollerMode.Adventure(mObelisk, 4000, [
					new CBackgroundObject(100, 370, "Bkg town.png"),
					new CBackgroundObject(4000 - 150, 370, "Bkg castle.png")
				]);
				break;
			case 6:
				mCurrentMode = new CTownMode(mObelisk, "Castle.png", "Act 2");
				break;
			case 7:
				mTextboxText = "King. Our town is under constant attack from\nthe monsters. Can you help us?";
				mTextboxAvatar = "Avatar player.png";
				break;
			case 8:
				mTextboxText = "The land is overrun with monsters and my\nmilitary commander is missing.";
				mTextboxAvatar = "Avatar king.png";
				break;
			case 9:
				mTextboxText = "The Royal Mage has discovered the source of\nthe monsters in a cave not far from here.";
				break;
			case 10:
				mTextboxText = "You seem like a strong warrior. Perhaps you\ncan escort him to the cave and investigate.";
				break;
			case 11:
				mCurrentMode = CSideScrollerMode.Adventure(mObelisk, 6000, [
					new CBackgroundObject(150, 370, "Bkg castle.png"),
					new CBackgroundObject(6000 - 103, 452, "Bkg cave.png")
					]);
				break;
			case 12:
				mCurrentMode = new CTopDownMode(mObelisk, "Cave map 0.png");
				break;
			case 13:
				mCurrentMode = new CTownMode(mObelisk, "Tiny world.png");
				mTextboxText = "What is THAT?";
				mTextboxAvatar = "Avatar player.png";
				break;
			case 14:
				mTextboxText = "That is the source of the monsters.";
				mTextboxAvatar = "Avatar mage.png";
				break;
			case 15:
				mTextboxText = "They have grown too big for their world and\nso they have come to claim ours.";
				break;
			case 16:
				mTextboxText = "Perhaps I can cast a spell to get rid of them.";
				break;
			case 17:
				mTextboxText = "...";
				mTextboxAvatar = "Avatar player.png";
				break;
			case 18:
				mTextboxText = "Well that didn't work.";
				mTextboxAvatar = "Avatar mage.png";
				break;
			case 19:
				mTextboxText = "Look out! *CRASH*";
				break;
			case 20:
				mTextboxText = "I think I made it mad! Let's get out of here\nnow! Hopefully there's another way out!";
				break;
			case 21:
				mCurrentMode = new CTopDownMode(mObelisk, "Cave map 1.png");
				break;
			case 22:
			case 23:
				mCurrentMode = CSideScrollerMode.Adventure(mObelisk, 8000, [
					new CBackgroundObject(103, 452, "Bkg cave exit.png"),
					new CBackgroundObject(8000 - 140, 380, "Bkg tower.png")
					]);
				if (mCheckpoint == 22)
				{
					mTextboxText = "Let's go back to my tower so I can prepare\nanother spell to get rid of that thing.";
					mTextboxAvatar = "Avatar mage.png";
				}
				break;
			case 24:
				mCurrentMode = new CTopDownMode(mObelisk, "Tower map 0.png");
				break;
			case 25:
				mCurrentMode = new CTopDownMode(mObelisk, "Tower map 1.png");
				break;
			case 26:
				mCurrentMode = new CTopDownMode(mObelisk, "Tower map 2.png");
				break;
			case 27:
				mCurrentMode = new CTopDownMode(mObelisk, "Tower map 3.png");
				break;
			case 28:
				mCurrentMode = new CTownMode(mObelisk, "Tower.png");
				mTextboxText = "We made it. Now wait here while I prepare the\nspell...";
				mTextboxAvatar = "Avatar mage.png";
				break;
			case 29:
				mTextboxText = "...";
				break;
			case 30:
				mTextboxText = "Almost done...";
				break;
			case 31:
				mTextboxText = "...";
				break;
			case 32:
				mTextboxText = "Now, stand back!";
				break;
			case 33:
				mCrazyTime = 0;
				break;
			case 34:
				mTextboxText = "It worked! All the monsters are gone!\nThe world is safe again!";
				break;
			case 35:
				mObelisk.SetBackgroundColour(new CColour(0, 0, 0));
				mWin = true;
				mCurrentMode = null;
				break;
		}
	}



	public void GameOver()
	{
		mObelisk.SetBackgroundColour(new CColour(0, 0, 0));
		mGameOver = true;
	}
}