﻿module CMainController;
import std.math, std.string;
import win32.windows;
import Obelisk.CColour, Obelisk.EKey, Obelisk.IMainController, Obelisk.IObelisk, Obelisk.IThread;
import CGameController;

extern (C) IMainController CreateMainController(IObelisk aObelisk)
{
	return new CMainController(aObelisk);
}

class CMainController : IMainController
{
	private IObelisk mObelisk;
	private CGameController mGameController;
	private bool mPaused = false;

	public this(IObelisk aObelisk)
	{
		aObelisk.SetCaption("Incursion");
		aObelisk.SetSize(800, 600);
		aObelisk.SetShaders(false);
		aObelisk.SetAudio(false);
		mObelisk = aObelisk;
	}



	public void Initialise()
	{
		mObelisk.LoadFont("Berylium.ttf");
		mObelisk.LoadTexture("Textbox.png");
		mObelisk.LoadTexture("Avatar player.png");
		mObelisk.LoadTexture("Avatar person.png");
		mObelisk.LoadTexture("Avatar king.png");
		mObelisk.LoadTexture("Avatar mage.png");
		mObelisk.LoadTexture("Town.png");
		mObelisk.LoadTexture("Castle.png");
		mObelisk.LoadTexture("Tiny world.png");
		mObelisk.LoadTexture("Tower.png");
		mObelisk.LoadTexture("Sword.png");
		for (int i = 0; i < 4; i++)
		{
			mObelisk.LoadTexture(format("Player right %d.png", i));
			mObelisk.LoadTexture(format("Player left %d.png", i));
			mObelisk.LoadTexture(format("Monster right %d.png", i));
			mObelisk.LoadTexture(format("Monster left %d.png", i));
		}
		mObelisk.LoadTexture("Player up.png");
		mObelisk.LoadTexture("Player down.png");
		mObelisk.LoadTexture("Bkg town.png");
		mObelisk.LoadTexture("Bkg castle.png");
		mObelisk.LoadTexture("Bkg cave.png");
		mObelisk.LoadTexture("Bkg cave exit.png");
		mObelisk.LoadTexture("Bkg tower.png");
		for (int i = 0; i < 6; i++)
			mObelisk.LoadTexture(format("Bkg cloud %d.png", i));

		//mObelisk.LoadMusic("Quiet town.mp3");

		mObelisk.ProjectionUI();
		mObelisk.StartTimer();
	}



	public void OnNotify(int aMessage, int aData)
	{
	}



	public void OnTimer()
	{
		if (mGameController is null)
			RenderTitleScreen();
		else
		{
			if (!mPaused)
				mGameController.Update();
			mObelisk.BeginFrame();
			mGameController.Render();
			mObelisk.EndFrame();
		}
	}



	private int mAlphaCount = 0;
	private int mColourCount = 0;

	private float Interpolate(float aV0, float aV1, float aNumerator, float aDenominator)
	{
		float aDelta = aNumerator / aDenominator;
		return aV0 * (1 - aDelta) + aV1 * aDelta;
	}

	private void RenderTitleScreen()
	{
		if (mObelisk.IsKeyPressed(EKey.Enter))
			mGameController = new CGameController(mObelisk);

		mAlphaCount = (mAlphaCount + 1) % (50 * 6);
		mColourCount = (mColourCount + 1) % (50 * 8);

		mObelisk.BeginFrame();

		mObelisk.DrawSprite(mObelisk.GetTexture("Tiny world.png"), 0, 0);

		CColour Colour = new CColour(0xEFC9B8);

		mObelisk.TextBegin(400, 180, 0.5);
		mObelisk.TextSetFont("Berylium.ttf", 160);
		mObelisk.SetColour(new CColour(
			Interpolate(Colour.R, 1, cos(mColourCount * (PI * 2) / (50 * 8)) / 2 + 0.5, 1),
			Interpolate(Colour.G, 1, cos(mColourCount * (PI * 2) / (50 * 8)) / 2 + 0.5, 1),
			Interpolate(Colour.B, 1, cos(mColourCount * (PI * 2) / (50 * 8)) / 2 + 0.5, 1)));
		mObelisk.SetAlpha(Interpolate(0.8, 1, cos(mAlphaCount * (PI * 2) / (50 * 6)) / 2 + 0.5, 1));
		mObelisk.TextWrite("Incursion");
		mObelisk.TextEnd();
		mObelisk.SetAlpha(1);
		mObelisk.SetColour(new CColour(1, 1, 1));

		mObelisk.TextBegin(400, 520, 0.5);
		mObelisk.TextSetFont("Berylium.ttf", 50);
		mObelisk.TextWrite("Press Enter to start");
		mObelisk.TextEnd();

		mObelisk.EndFrame();
	}



	public void OnDeactivate()
	{
		mPaused = true;
	}



	public void OnActivate()
	{
		mPaused = false;
	}



	public void OnKeyDown(EKey aKey)
	{
		if (aKey == EKey.Escape)
			mObelisk.Quit();
	}



	public void OnKeyUp(EKey aKey)
	{
	}



	public void ThreadFinished(IThread aThread)
	{
	}



	public void FormRenderButton(int aX0, int aY0, int aX1, int aY1, int aX, int aY,
			string aText, bool aHover)
	{
	}
}