module CSideScrollerMode;
import std.algorithm, std.math, std.random, std.string;
import DOpenGL.gl;
import Obelisk.CColour, Obelisk.CVector, Obelisk.EKey, Obelisk.IObelisk;
import IGameMode;

	CSideScrollerMode TownCombat(IObelisk aObelisk, int aNumMonsters, bool aShowHelp,
			CBackgroundObject[] aBackground)
	{
		return new CSideScrollerMode(aObelisk, true, 800, aNumMonsters, aShowHelp, aBackground);
	}

	CSideScrollerMode Adventure(IObelisk aObelisk, int aSize, CBackgroundObject[] aBackground)
	{
		return new CSideScrollerMode(aObelisk, false, aSize, -1, false, aBackground);
	}

class CBackgroundObject
{
	public int X;
	public int Y;
	public string Texture;

	public this(int aX, int aY, string aTexture)
	{
		X = aX;
		Y = aY;
		Texture = aTexture;
	}
}

class CSideScrollerMode : IGameMode
{
	private const int cSwordSwingDuration = 11;
	private const int cSwordDuration = 15;
	private const int cPlayerDamageDuration = 1 * 50;
	private const int cMonsterDamageDuration = 1 * 50;

	private IObelisk mObelisk;
	private int mSize;
	private int mHelpTime;
	private CBackgroundObject[] mBackground;
	private int mCameraX = 400;

	private int mPlayerDirection = 1;
	private int mPlayerX;
	private int mPlayerAnimation = 0;
	private int mPlayerAnimationFrame;
	private int mSwordFrame = -1;
	private int mPlayerHP = 5;
	private int mPlayerDamageTime = -1;

	private CMonster[] mMonsters;
	private int mSpawnCounter;
	private int mNumMonsters;
	private int mSideCounter = 0;

	/*
	public static CSideScrollerMode TownCombat(IObelisk aObelisk, int aNumMonsters)
	{
		return new CSideScrollerMode(aObelisk, true, 800, aNumMonsters);
	}

	public static CSideScrollerMode Adventure(IObelisk aObelisk, int aSize)
	{
		return new CSideScrollerMode(aObelisk, false, aSize, -1);
	}
	*/

	public this(IObelisk aObelisk, bool aCenter, int aSize, int aNumMonsters, bool aShowHelp,
			CBackgroundObject[] aBackground)
	{
		mObelisk = aObelisk;
		mSize = aSize;
		mNumMonsters = aNumMonsters;
		mHelpTime = aShowHelp ? 0 : -1;
		mBackground = aBackground;
		int NumClouds = mSize / 400;
		for (int i = 0; i < NumClouds; i++)
			mBackground = [ new CBackgroundObject(uniform(0, mSize), uniform(90, 260),
					format("Bkg cloud %d.png", uniform(0, 6))) ] ~ mBackground;

		mObelisk.SetBackgroundColour(new CColour(0xA9DAE5));

		mPlayerX = aCenter ? 400 : 150;
		mSpawnCounter = 2 * 50;
	}



	public void Update(CGameController aGame)
	{
		if (mObelisk.IsKeyDown(EKey.Backspace) && mObelisk.IsKeyDown(EKey.Tab))
		{
			aGame.GameNext();
			return;
		}
		int PlayerMoveX = 0;
		if (mObelisk.IsKeyDown(EKey.Left))
			PlayerMoveX = -4;
		else if (mObelisk.IsKeyDown(EKey.Right))
			PlayerMoveX = 4;
		if (mSwordFrame == -1 && mPlayerDamageTime == -1 && mObelisk.IsKeyDown(EKey.Space))
			mSwordFrame = 0;

		if (PlayerMoveX == 0)
		{
			mPlayerAnimation = 0;
			mPlayerAnimationFrame = 0;
		}
		else
		{
			mPlayerDirection = sgn(PlayerMoveX);
			mPlayerX += PlayerMoveX;
			if (mPlayerX < 0)
				mPlayerX = 0;
			if (mPlayerX > mSize)
				mPlayerX = mSize;
			if (mNumMonsters == -1 && mPlayerX > mSize - 200)
			{
				aGame.GameNext();
				return;
			}

			if (mPlayerAnimation != 1)
			{
				mPlayerAnimation = 1;
				mPlayerAnimationFrame = 7;
			}
			else
				mPlayerAnimationFrame = (mPlayerAnimationFrame + 1) % (4 * 7);
		}
		if (mSwordFrame != -1)
		{
			if (++mSwordFrame == cSwordDuration)
				mSwordFrame = -1;
		}
		if (mPlayerDamageTime != -1)
		{
			if (++mPlayerDamageTime == cPlayerDamageDuration)
				mPlayerDamageTime = -1;
		}

		mCameraX = mPlayerX;
		if (mCameraX > mSize - 400)
			mCameraX = mSize - 400;
		if (mCameraX < 400)
			mCameraX = 400;

		if (mNumMonsters != 0 && --mSpawnCounter == 0)
		{
			int Side = 1;
			if (++mSideCounter == (mNumMonsters == -1 ? 7 : 2))
			{
				mSideCounter = 0;
				Side = 0;
			}
			int X = mCameraX + (Side == 0 ? -420 : 420);
			int Direction = Side == 0 ? 1 : -1;
			mMonsters ~= [ new CMonster(X, Direction, uniform(0, 4 * 7)) ];
			if (mNumMonsters != -1)
				mNumMonsters--;
			mSpawnCounter = 3 * 50;
		}

		for (int i = mMonsters.length - 1; i >= 0; i--)
		{
			CMonster Monster = mMonsters[i];
			if (Monster.DamageTime == -1)
			{
				if (mSwordFrame >= cSwordSwingDuration)
				{
					int SwordX0 = mPlayerDirection == 1 ? mPlayerX + 12 : mPlayerX - 37;
					int SwordX1 = mPlayerDirection == 1 ? mPlayerX + 37 : mPlayerX - 12;
					if (Monster.X >= SwordX0 && Monster.X < SwordX1)
					{
						if (--Monster.HP == 0)
						{
							for (int j = i; j < mMonsters.length - 1; j++)
								mMonsters[j] = mMonsters[j + 1];
							mMonsters.length = mMonsters.length - 1;
							if (mMonsters.length == 0 && mNumMonsters == 0)
							{
								aGame.GameNext();
								return;
							}
							continue;
						}
						Monster.DamageTime = 0;
						Monster.XVelocity = mPlayerDirection * 15;
					}
				}
				if (mPlayerDamageTime == -1 && abs(Monster.X - mPlayerX) <= 19)
				{
					if (--mPlayerHP == 0)
					{
						aGame.GameOver();
						return;
					}
					mPlayerDamageTime = 0;
					// commented this out because it makes it too hard
					//mSwordFrame = -1;
				}
			}

			bool CheckCollisions = true;
			float MoveX = 0;
			if (Monster.DamageTime == -1)
			{
				if (Monster.X > mPlayerX)
				{
					Monster.Direction = -1;
					MoveX = -2;
				}
				else
				{
					Monster.Direction = 1;
					MoveX = 2;
				}
			}
			else
			{
				MoveX = Monster.XVelocity;
				Monster.XVelocity *= 0.9;
				CheckCollisions = false;
			}

			if (MoveX != 0 && CheckCollisions)
			{
				for (int j = i + 1; j < mMonsters.length; j++)
				{
					if (mMonsters[j].DamageTime != -1)
						continue;
					if (abs(mMonsters[j].X - (mMonsters[i].X + MoveX)) < 21)
					{
						MoveX = 0;
						break;
					}
				}
			}
			if (MoveX != 0)
			{
				Monster.X += MoveX;
				Monster.Frame = (Monster.Frame + 1) % (4 * 7);
			}
			if (Monster.DamageTime != -1)
			{
				if (++Monster.DamageTime == cMonsterDamageDuration)
					Monster.DamageTime = -1;
			}
		}
	}



	public void Render()
	{
		mObelisk.SetLinearFiltering(false);

		// draw background
		mObelisk.QuadColour(new CVector(0, 320), new CVector(800, 320), new CVector(800, 600),
			new CVector(0, 600), new CColour(0x52A033));

		glPushMatrix();
		glTranslatef(-(mCameraX - 400), 0, 0);

		mObelisk.SetSpriteAlign(true);

		foreach (CBackgroundObject Background; mBackground)
			mObelisk.DrawSprite(mObelisk.GetTexture(Background.Texture), Background.X,
					Background.Y, -1, -1, 0.5, 1);

		// draw monsters
		foreach (CMonster Monster; mMonsters)
		{
			bool ShowDamage = Monster.DamageTime != -1 && Monster.DamageTime / 3 % 2 == 0;
			if (ShowDamage)
				mObelisk.SetColour(new CColour(1, 0, 0));
			int Frame = Monster.DamageTime != -1 ? 0 : Monster.Frame / 7;
			mObelisk.DrawSprite(mObelisk.GetTexture(format("Monster %s %d.png",
					Monster.Direction == 1 ? "right" : "left", Frame)),
					cast(int)Monster.X, 420, -1, -1, 0.5, 1);
			if (ShowDamage)
				mObelisk.SetColour(new CColour(1, 1, 1));
		}

		// draw player;
		glPushMatrix();
		glTranslatef(mPlayerX, 420, 0);
		bool ShowDamage = mPlayerDamageTime != -1 && mPlayerDamageTime / 3 % 2 == 0;
		if (ShowDamage)
			mObelisk.SetColour(new CColour(1, 0, 0));
		mObelisk.DrawSprite(mObelisk.GetTexture(format("Player %s %d.png",
				mPlayerDirection == 1 ? "right" : "left", mPlayerAnimationFrame / 7)),
				0, 0, -1, -1, 0.5, 1);
		float SwordAngle;
		if (mSwordFrame == -1)
			SwordAngle = (mPlayerDirection == 1 ? 35 : (180 - 35)) * PI / 180;
		else
		{
			float StartAngle = mPlayerDirection == 1 ? -PI * 3 / 4 : -PI / 4;
			float EndAngle = mPlayerDirection == 1 ? 0 : -PI;
			SwordAngle = Interpolate(StartAngle, EndAngle, min(mSwordFrame,
					cSwordSwingDuration), cSwordSwingDuration);
		}
		glPushMatrix();
		glTranslatef(0, -25, 0);
		glRotatef(SwordAngle * 180 / PI, 0, 0, 1);
		mObelisk.QuadTexture(new CVector(0, -5), 0, 0, new CVector(37, -5), 1, 0,
				new CVector(37, 5), 1, 1, new CVector(0, 5), 0, 1,
				mObelisk.GetTexture("Sword.png"));
		glPopMatrix();
		glPopMatrix();
		if (ShowDamage)
			mObelisk.SetColour(new CColour(1, 1, 1));
		glPopMatrix();

		// draw UI
		mObelisk.SetColour(new CColour(0.3, 0.3, 0.3));
		mObelisk.SetAlpha(0.7);
		mObelisk.TextBegin(33, 53, 0);
		mObelisk.TextSetFont("Berylium.ttf", 40);
		mObelisk.TextWrite(format("HP: %d", mPlayerHP));
		mObelisk.TextEnd();
		mObelisk.SetColour(new CColour(1, 1, 1));
		mObelisk.SetAlpha(1);
		mObelisk.TextBegin(30, 50, 0);
		mObelisk.TextSetFont("Berylium.ttf", 40);
		mObelisk.TextWrite(format("HP: %d", mPlayerHP));
		mObelisk.TextEnd();

		if (mHelpTime != -1)
		{
			mObelisk.SetColour(new CColour(0.3, 0.3, 0.3));
			mObelisk.SetAlpha(0.7);
			mObelisk.TextBegin(403, 283, 0.5);
			mObelisk.TextSetFont("Berylium.ttf", 40);
			mObelisk.TextWrite("Press Space to attack");
			mObelisk.TextEnd();
			mObelisk.SetColour(new CColour(1, 1, 1));
			mObelisk.SetAlpha(1);
			mObelisk.TextBegin(400, 280, 0.5);
			mObelisk.TextSetFont("Berylium.ttf", 40);
			mObelisk.TextWrite("Press Space to attack");
			mObelisk.TextEnd();
			if (++mHelpTime == 50 * 3)
				mHelpTime = -1;
		}
	}



	private float Interpolate(float aV0, float aV1, float aNumerator, float aDenominator)
	{
		float aDelta = aNumerator / aDenominator;
		return aV0 * (1 - aDelta) + aV1 * aDelta;
	}
}



class CMonster
{
	public float X;
	public float XVelocity;
	public int Direction;
	public int Frame;
	public int DamageTime = -1;
	public int HP = 2;

	public this(int aX, int aDirection, int aFrame)
	{
		X = aX;
		XVelocity = 0;
		Direction = aDirection;
		Frame = aFrame;
	}
}