module CTopDownMode;
import std.algorithm, std.math, std.string;
import DOpenGL.gl;
import Obelisk.CColour, Obelisk.CGrid, Obelisk.CVector, Obelisk.EKey, Obelisk.IObelisk;
import CGameController, IGameMode;

class CMonster
{
	public int X;
	public int Y;
	public int Direction = 0;
	public bool Awake = false;
	private int MoveCounter = -1;

	public this(int aX, int aY)
	{
		X = aX;
		Y = aY;
	}
}

class CTopDownMode : IGameMode
{
	private const int cSwordSwingDuration = 11;
	private const int cSwordDuration = 15;

	private IObelisk mObelisk;
	private CGrid mGrid;

	private int mCameraX;
	private int mCameraY;

	private int mPlayerX;
	private int mPlayerY;
	private int mPlayerDirection = 2;
	private int mMoveCounter = 0;
	private int mPlayerHP = 5;
	private int mPlayerDamageTime = -1;
	private int mEndX;
	private int mEndY;
	private int mSwordFrame = -1;

	private CMonster[] mMonsters;

	public this(IObelisk aObelisk, string aGridName)
	{
		mObelisk = aObelisk;
		mObelisk.SetBackgroundColour(new CColour(0x000000));
		int[int] ColourMap;
		ColourMap[0xB2B2B2] = 0;
		ColourMap[0x5B5B5B] = 1;
		ColourMap[0x000000] = 2;
		ColourMap[0xff0000] = 3;
		ColourMap[0xffff00] = 4;
		ColourMap[0xB2B200] = 5;
		mGrid = mObelisk.LoadGrid(aGridName, ColourMap);
		for (int y = 0; y < mGrid.GetYSize(); y++)
		{
			for (int x = 0; x < mGrid.GetXSize(); x++)
			{
				int Data = mGrid.Get(x, y);
				if (Data == 3)
				{
					mPlayerX = x;
					mPlayerY = y;
					mGrid.Set(x, y, 0);
				}
				else if (Data == 4)
				{
					mEndX = x;
					mEndY = y;
					mGrid.Set(x, y, 0);
				}
				else if (Data == 5)
				{
					mMonsters ~= [ new CMonster(x, y) ];
					mGrid.Set(x, y, 0);
				}
			}
		}
	}



	public void Update(CGameController aGame)
	{
		mMoveCounter = (mMoveCounter + 1) % 5;

		{
			int MoveX = 0;
			int MoveY = 0;
			if (mObelisk.IsKeyDown(EKey.Left))
			{
				mPlayerDirection = 0;
				MoveX = -1;
			}
			else if (mObelisk.IsKeyDown(EKey.Right))
			{
				mPlayerDirection = 1;
				MoveX = 1;
			}
			else if (mObelisk.IsKeyDown(EKey.Up))
			{
				mPlayerDirection = 2;
				MoveY = -1;
			}
			else if (mObelisk.IsKeyDown(EKey.Down))
			{
				mPlayerDirection = 3;
				MoveY = 1;
			}
			if (mMoveCounter == 0)
			{
				if (mGrid.Get(mPlayerX + MoveX, mPlayerY + MoveY) == 0)
				{
					bool CanMove = true;
					foreach (CMonster Monster; mMonsters)
					{
						if (mPlayerX + MoveX == Monster.X && mPlayerY + MoveY == Monster.Y)
						{
							CanMove = false;
							break;
						}
					}
					if (CanMove)
					{
						mPlayerX += MoveX;
						mPlayerY += MoveY;
						if (mPlayerX == mEndX && mPlayerY == mEndY)
						{
							aGame.GameNext();
							return;
						}
						mMoveCounter = 0;
					}
				}
			}
		}
		if (mSwordFrame != -1)
		{
			if (mSwordFrame >= cSwordSwingDuration)
			{
				int AttackX = mPlayerX;
				int AttackY = mPlayerY;
				final switch (mPlayerDirection)
				{
					case 0:
						AttackX--;
						break;
					case 1:
						AttackX++;
						break;
					case 2:
						AttackY--;
						break;
					case 3:
						AttackY++;
						break;
				}
				for (int i = mMonsters.length - 1; i >= 0; i--)
				{
					CMonster Monster = mMonsters[i];
					if (AttackX == Monster.X && AttackY == Monster.Y)
					{
						for (int j = i; j < mMonsters.length - 1; j++)
							mMonsters[j] = mMonsters[j + 1];
						mMonsters.length = mMonsters.length - 1;
					}
				}
			}
			if (++mSwordFrame == cSwordDuration)
				mSwordFrame = -1;
		}
		if (mSwordFrame == -1 && mObelisk.IsKeyDown(EKey.Space))
			mSwordFrame = 0;
		if (mPlayerDamageTime != -1 && ++mPlayerDamageTime == 50)
			mPlayerDamageTime = -1;

		mCameraX = mPlayerX;
		mCameraY = mPlayerY;

		foreach (int i, CMonster Monster; mMonsters)
		{
			if (!Monster.Awake)
			{
				if ((Monster.X - mPlayerX) ^^ 2 + (Monster.Y - mPlayerY) ^^ 2 <= 7 ^^ 2)
					Monster.Awake = true;
			}
			else if (mMoveCounter == 0 && Monster.MoveCounter != -1)
			{
				if (++Monster.MoveCounter == 3)
					Monster.MoveCounter = -1;
			}
			else if (mMoveCounter == 0 && Monster.MoveCounter == -1)
			{
				int MoveX = 0;
				int MoveY = 0;
				int DeltaX = mPlayerX - Monster.X;
				int DeltaY = mPlayerY - Monster.Y;
				for (int q = 0; q < 2; q++)
				{
					if (abs(DeltaX) > abs(DeltaY))
					{
						MoveX = sgn(DeltaX);
						Monster.Direction = MoveX == -1 ? 0 : 1;
					}
					else
						MoveY = sgn(DeltaY);
					if (Monster.X + MoveX == mPlayerX && Monster.Y + MoveY == mPlayerY)
					{
						if (mPlayerDamageTime == -1 && --mPlayerHP == 0)
						{
							aGame.GameOver();
							return;
						}
						else if (mPlayerDamageTime == -1)
							mPlayerDamageTime = 0;
						break;
					}
					else if (mGrid.Get(Monster.X + MoveX, Monster.Y + MoveY) == 0)
					{
						bool CanMove = true;
						foreach (int j, CMonster Monster1; mMonsters)
						{
							if (i != j && Monster.X + MoveX == Monster1.X && Monster.Y + MoveY == Monster1.Y)
							{
								CanMove = false;
								break;
							}
						}
						if (CanMove)
						{
							Monster.X += MoveX;
							Monster.Y += MoveY;
							Monster.MoveCounter = 0;
							break;
						}
					}
					if (MoveX != 0)
						DeltaX = 0;
					else
						DeltaY = 0;
				}
			}
		}
	}



	public void Render()
	{
		int CameraX = (mCameraX * 30 - 400);
		int CameraY = (mCameraY * 30 - 300);

		CColour[] Colour = [ new CColour(0xB2B2B2), new CColour(0x5B5B5B),
				new CColour(0x000000) ];
		auto v0 = new CVector(0, 0);
		auto v1 = new CVector(0, 0);
		auto v2 = new CVector(0, 0);
		auto v3 = new CVector(0, 0);
		for (int y = 0; y < mGrid.GetYSize(); y++)
		{
			for (int x = 0; x < mGrid.GetXSize(); x++)
			{
				v0.X = x * 30 - 15 - CameraX;
				v0.Y = y * 30 - 15 - CameraY;
				v1.X = x * 30 + 15 - CameraX;
				v1.Y = y * 30 - 15 - CameraY;
				v2.X = x * 30 + 15 - CameraX;
				v2.Y = y * 30 + 15 - CameraY;
				v3.X = x * 30 - 15 - CameraX;
				v3.Y = y * 30 + 15 - CameraY;
				if (v1.X < 0 || v0.X > 800 || v2.Y < 0 || v0.Y > 600)
					continue;
				mObelisk.QuadColour(v0, v1, v2, v3, Colour[mGrid.Get(x, y)]);
			}
		}

		glPushMatrix();
		glTranslatef(-CameraX, -CameraY, 0);

		foreach (CMonster Monster; mMonsters)
		{
			string MonsterTexture = Monster.Direction == 0 ? "Monster left 0.png" : "Monster right 0.png";
			mObelisk.DrawSprite(mObelisk.GetTexture(MonsterTexture), Monster.X * 30, Monster.Y * 30,
					-1, -1, 0.5, 1);
		}

		string PlayerTexture;
		int SwordOffsetX = 0;
		float SwordNormalAngle;
		float SwordSwingStartAngle;
		float SwordSwingEndAngle;
		final switch (mPlayerDirection)
		{
			case 0:
				PlayerTexture = "Player left 0.png";
				SwordNormalAngle = (180 - 35) * PI / 180;
				SwordSwingStartAngle = -PI / 4;
				SwordSwingEndAngle = -PI;
				break;
			case 1:
				PlayerTexture = "Player right 0.png";
				SwordNormalAngle = 35 * PI / 180;
				SwordSwingStartAngle = -PI * 3 / 4;
				SwordSwingEndAngle = 0;
				break;
			case 2:
				PlayerTexture = "Player up.png";
				SwordOffsetX = 6;
				SwordNormalAngle = -PI / 2;
				SwordSwingStartAngle = PI / 4;
				SwordSwingEndAngle = -PI / 2;
				break;
			case 3:
				PlayerTexture = "Player down.png";
				SwordOffsetX = -6;
				SwordNormalAngle = PI / 2;
				SwordSwingStartAngle = PI * 3 / 2;
				SwordSwingEndAngle = PI / 2;
				break;
		}
		bool ShowDamage = mPlayerDamageTime != -1 && mPlayerDamageTime / 3 % 2 == 0;
		if (ShowDamage)
			mObelisk.SetColour(new CColour(1, 0, 0));
		mObelisk.DrawSprite(mObelisk.GetTexture(PlayerTexture), mPlayerX * 30, mPlayerY * 30,
				-1, -1, 0.5, 1);
		float SwordAngle;
		if (mSwordFrame == -1)
			SwordAngle = SwordNormalAngle;
		else
			SwordAngle = Interpolate(SwordSwingStartAngle, SwordSwingEndAngle, min(mSwordFrame,
					cSwordSwingDuration), cSwordSwingDuration);
		glPushMatrix();
		glTranslatef(mPlayerX * 30 + SwordOffsetX, mPlayerY * 30, 0);
		glTranslatef(0, -25, 0);
		glRotatef(SwordAngle * 180 / PI, 0, 0, 1);
		mObelisk.QuadTexture(new CVector(0, -5), 0, 0, new CVector(37, -5), 1, 0,
				new CVector(37, 5), 1, 1, new CVector(0, 5), 0, 1,
				mObelisk.GetTexture("Sword.png"));
		glPopMatrix();
		if (ShowDamage)
			mObelisk.SetColour(new CColour(1, 1, 1));

		glPopMatrix();

		mObelisk.SetColour(new CColour(0.3, 0.3, 0.3));
		mObelisk.SetAlpha(0.7);
		mObelisk.TextBegin(33, 53, 0);
		mObelisk.TextSetFont("Berylium.ttf", 40);
		mObelisk.TextWrite(format("HP: %d", mPlayerHP));
		mObelisk.TextEnd();
		mObelisk.SetColour(new CColour(1, 1, 1));
		mObelisk.SetAlpha(1);
		mObelisk.TextBegin(30, 50, 0);
		mObelisk.TextSetFont("Berylium.ttf", 40);
		mObelisk.TextWrite(format("HP: %d", mPlayerHP));
		mObelisk.TextEnd();
	}



	private float Interpolate(float aV0, float aV1, float aNumerator, float aDenominator)
	{
		float aDelta = aNumerator / aDenominator;
		return aV0 * (1 - aDelta) + aV1 * aDelta;
	}
}